/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file sim.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief header file for SIM utilities. 
*
*******************************************************************************
*
* provide APIs for accessing SIM
******************************************************************************/

#ifndef SIM_H_
#define SIM_H_

/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* define SIM device ID types
*
*//*! @addtogroup sim_id_types
* @{
*******************************************************************************/

typedef enum {
    ID_TYPE_FAMID,                  /*!< device Family ID */
    ID_TYPE_SUBFAMID,               /*!< device Subfamily ID */ 
    ID_TYPE_REVID,                  /*!< device Revision ID */
    ID_TYPE_PINID                   /*!< device Pin ID (Pin count) */
} IDType;
/*! @} End of sim_id_types                                                    					*/

/******************************************************************************
* Macros
******************************************************************************/ 

/******************************************************************************
* Types
******************************************************************************/
    
/* SIM configuration structure 
 */ 
 
/******************************************************************************
* define SIM configuration structure
*
*//*! @addtogroup sim_config_type
* @{
*******************************************************************************/

/*!
 * @brief SIM configuration structure.
 *
 */
typedef struct{
    struct{
	uint32_t bEnableCLKOUT  : 1;        /*!< 1: enable , 0: disable */
	uint32_t bTXDME         : 1;        /*!< 1: enable TXDME, 0: disable */
	uint32_t bFTMSYNC       : 1;        /*!< 1: enable FTM SYNC, 0: no sync */
	uint32_t bRXDFE         : 1;        /*!< 1: enable RXD filter, 0: no filter */
	uint32_t bRXDCE         : 1;        /*!< 1: enable RXD capture, 0: no capture */
	uint32_t bACIC          : 1;        /*!< 1: ACMP0 to FTM1 channel0 connection, 0: no connection */
	uint32_t bRTCC          : 1;        /*!< 1: RTC overflow connected to FTM1 channel1, 0: no connection */
	uint32_t u8ADHWT        : 2;        /*!< ADC h/w trigger source selection */
	uint32_t bDisableSWD    : 1;        /*!< 1: disable SWD, 0: enable */
	uint32_t bDisableRESET  : 1;        /*!< 1: disable RESET pin, 0: enable */
	uint32_t bDisableNMI    : 1;        /*!< 1: disable NMI pin, 0: enable */
    uint32_t bBusDiv        : 1;        /*!< bus divider BUSDIV value */
    } sBits;
    uint8_t     u8Delay;                /*!< delay value */
    uint8_t     u8BusRef;               /*!< bus reference */
    uint32_t    u32PinSel;              /*!< pin select reg value */
    uint32_t    u32SCGC;                /*!< clock gating value register */   
} SIM_ConfigType, *SIM_ConfigPtr; /*!< sim configuration structure type */
/*! @} End of sim_config_type                                                    					*/


  
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/
/******************************************************************************
* define SIM API list
*
*//*! @addtogroup sim_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief enable clock output.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see  SIM_DisableClockOutput
*****************************************************************************/
__STATIC_INLINE void SIM_EnableClockOutput(void)
{
    SIM->SOPT |= (SIM_SOPT_CLKOE_MASK);    
}

/*****************************************************************************//*!
*
* @brief disable clock output.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see  SIM_EnableClockOutput
*****************************************************************************/
__STATIC_INLINE void SIM_DisableClockOutput(void)
{
    SIM->SOPT &= ~(SIM_SOPT_CLKOE_MASK);    
}

#if defined(CPU_KE02) 
/*****************************************************************************//*!
*
* @brief set bus divide BUSDIV.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_SetBusDivide(uint8_t u8Divide)
{
    SIM->BUSDIV = u8Divide;    
}
#endif

/*****************************************************************************//*!
*
* @brief remap FTM2CH3 pin from default to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM2CH3Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM2PS3_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap FTM2CH2 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM2CH2Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM2PS2_MASK;    
}
#if defined(CPU_KE02) 
/*****************************************************************************//*!
*
* @brief remap FTM2CH1 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM2CH1Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM2PS1_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap FTM2CH0 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM2CH0Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM2PS0_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap FTM1CH1 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM1CH1Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM1PS1_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap FTM1CH1 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapFTM1CH0Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_FTM1PS0_MASK;    
}
#endif
/*****************************************************************************//*!
*
* @brief remap UART0 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapUART0Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_UART0PS_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap SPI0 pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapSPI0Pin(void)
{
    SIM->PINSEL |= SIM_PINSEL_SPI0PS_MASK;    
}

/*****************************************************************************//*!
*
* @brief remap I2C pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapI2CPin(void)
{
    SIM->PINSEL |= SIM_PINSEL_IICPS_MASK;    
}
#if defined(CPU_KE02) 
/*****************************************************************************//*!
*
* @brief remap RTC pin from default  to the other. 
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_RemapRTCPin(void)
{
    SIM->PINSEL |= SIM_PINSEL_RTCPS_MASK;    
}
#endif
/*****************************************************************************//*!
*
* @brief set bus clock output divide. 
*        
* @param[in]  u8Divide     divide (3-bits)
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_SetClockOutputDivide(uint8_t u8Divide)
{
    SIM->SOPT = (SIM->SOPT & ~(SIM_SOPT_BUSREF_MASK)) | SIM_SOPT_BUSREF(u8Divide & 0x07);    
}

/*****************************************************************************//*!
*
* @brief delay FTM2 triggering ADC for u8Delay bus clock output divide. 
*        
* @param[in]   u8Delay     delay value of Bus clock output divide.
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_DelayFTM2Trig2ADC(uint8_t u8Delay)
{
    SIM->SOPT = (SIM->SOPT & ~(SIM_SOPT_DELAY_MASK)) | SIM_SOPT_DELAY(u8Delay);    
}

/*****************************************************************************//*!
*
* @brief set ADC hardware trigger source to RTC overflow. 
*        
* @param  none        
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_TriggerADCByRTC(void)
{
    SIM->SOPT &= ~(SIM_SOPT_ADHWT_MASK);    
}

/*****************************************************************************//*!
*
* @brief set ADC hardware trigger source to PIT . 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_TriggerADCByPIT(void)
{
    SIM->SOPT = (SIM->SOPT & ~(SIM_SOPT_ADHWT_MASK)) | SIM_SOPT_ADHWT(1);    
}


/*****************************************************************************//*!
*
* @brief set ADC hardware trigger source to FTM2 init trigger with 8-bit programmable delay. 
*        
* @param  none      
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_TriggerADCByFTM2Init(void)
{
    SIM->SOPT = (SIM->SOPT & ~(SIM_SOPT_ADHWT_MASK)) | SIM_SOPT_ADHWT(2);    
}

/*****************************************************************************//*!
*
* @brief set ADC hardware trigger source to FTM2 match trigger with 8-bit programmable delay. 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_TriggerADCByFTM2Match(void)
{
    SIM->SOPT = (SIM->SOPT & ~(SIM_SOPT_ADHWT_MASK)) | SIM_SOPT_ADHWT(3);    
}

/*****************************************************************************//*!
*
* @brief enable UART0 TX modulation. 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_EnableUART0Modulation(void)
{
    SIM->SOPT |= (SIM_SOPT_TXDME_MASK);    
}

/*****************************************************************************//*!
*
* @brief disable UART0 TX modulation. 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_DisableUART0Modulation(void)
{
    SIM->SOPT &= ~(SIM_SOPT_TXDME_MASK);    
}


/*****************************************************************************//*!
*
* @brief enable UART0 RX filter. 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_EnableUART0Filter(void)
{
    SIM->SOPT |= (SIM_SOPT_RXDFE_MASK);    
}

/*****************************************************************************//*!
+FUNCTION----------------------------------------------------------------
* @function name: SIM_DisableUART0Filter
*
* @brief disable UART0 RX filter. 
*        
* @param         
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_DisableUART0Filter(void)
{
    SIM->SOPT &= ~(SIM_SOPT_RXDFE_MASK);    
}

#if defined(CPU_KE02) 
/*****************************************************************************//*!
*
* @brief enable ACMP0 input capture to FTM1 input channel0. 
*        
* @param  none      
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_EnableACMP0InputCapture(void)
{
    SIM->SOPT |= (SIM_SOPT_ACIC_MASK);    
}


/*****************************************************************************//*!
*
* @brief disable ACMP0 input capture to FTM1 input channel0.
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_DisableACMP0InputCapture(void)
{
    SIM->SOPT &= ~(SIM_SOPT_ACIC_MASK);    
}

/*****************************************************************************//*!
*
* @brief enable RTC capture to FTM1 input channel1. 
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_EnableRTCCapture(void)
{
    SIM->SOPT |= (SIM_SOPT_RTCC_MASK);    
}

/*****************************************************************************//*!
*
* @brief disable RTC capture to FTM1 input channel1. 
*        
* @param  none      
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_DisableRTCCapture(void)
{
    SIM->SOPT &= ~(SIM_SOPT_RTCC_MASK);    
}
#endif
/*****************************************************************************//*!
*
* @brief generate a softare sync trigger to FTM2 module (trigger).
*        
* @param  none       
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void SIM_GenerateSoftwareTrig2FTM2(void)
{
    SIM->SOPT |= (SIM_SOPT_FTMSYNC_MASK);    
}

/*! @} End of sim_api_list                                                    					*/

void SIM_Init(SIM_ConfigType *pConfig);
void SIM_SetClockGating(uint32_t u32PeripheralMask, uint8_t u8GateOn);
uint32_t SIM_GetStatus(uint32_t u32StatusMask);
uint8_t SIM_ReadID(IDType sID);

#endif /* SIM_H_ */


